/*
Title:    AndyPi Sonic Ruler for Arduino Nano
Code by:  Andrew Wilsion http://andypi.co.uk
Date:     02/11/2014
Version:  1.0

HC-SR04 Ultrasonic Sensor:
  VCC:  - Arduino +5VDC
  Trig  - Arduino Pin 11
  Echo  - Arduino Pin 12
  GND   - Arduino GND
        
HD44780 16x2 AndyPi LCD Display:
  LCD RS pin - Arduino digital pin 7
  LCD E pin  - Arduino digital pin 6
  LCD D4 pin - Arduino digital pin 5
  LCD D5 pin - Arduino digital pin 4
  LCD D6 pin - Arduino digital pin 3
  LCD D7 pin - Arduino digital pin 2
  LCD GND    - Arduino GND
  LCD +V     - Arduino V+
  LCD A (LED)- Arduino V+
  (Note - Wiring for contrast contol potentiometer and Backlight GND is including in AndyPi LCD)
        
NOTE: The speed of sound in air varies with temperature and pressure (and therefore with altitute)
You can search for tables of this online, or even use the ultrasonic sensor and a known distance to calculate it for yourself
At sea level, 15C, the spee od sound is 340.3m/s
At 2000m above sea level where AndyPi lives, it is approx 332.5m/s
        
 */

#include <LiquidCrystal.h>
LiquidCrystal lcd(7, 6, 5, 4, 3, 2);

int trigPin = 11;               // Trig - green Jumper
int echoPin = 12;               // Echo - yellow Jumper
float duration, distance;       // declare varaibles as floating point numbers (aka uses a decimal point)
const float SPEEDOFSOUND=332.5; // set the speed of sound 
const int MAXSDISTANCE = 400;   // set the maximum distance in cm (this is the max the sensor can reliably measure
char distRounded[10];           // temporarily holds data f

void setup() {
  // set up the LCD's number of columns and rows:
  lcd.begin(16, 2);
  //Define inputs and outputs
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
}
 
void loop()
{
 
  // The sensor is triggered by a HIGH pulse of 10 or more microseconds.
  // Give a short LOW pulse beforehand to ensure a clean HIGH pulse:
  digitalWrite(trigPin, LOW);
  delayMicroseconds(5);
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
 
  // Read the signal from the sensor: a HIGH pulse whose
  // duration is the time (in microseconds) from the sending
  // of the ping to the reception of its echo off of an object.
  pinMode(echoPin, INPUT);
  duration = pulseIn(echoPin, HIGH);
 
  /* convert the time into a distance rememeber:
  a) the time is measured for the distance to the obejct and back, so it needs to be divided by 2
  b) the time is measure in microseconds so needs to be divided by 1000000 to convert to seconds
  c) the distance will output in metres, need to multiply by 100 to view in cm
  speed= distance/time
  distance = time*speed
  */
  distance = (((duration/1000000)/2)*SPEEDOFSOUND)*100;
  
  // convert the distance into a string without decimal places
  dtostrf(distance,1,0,distRounded); 
  
  if (distance > MAXSDISTANCE) {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Sonic Ruler");
    lcd.setCursor(0, 1);
    lcd.print("Exceeds 4 metres");
  }
  else {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Sonic Ruler");
    lcd.setCursor(0, 1);
    lcd.print(distRounded);
    lcd.setCursor(4, 1);
    lcd.print("cm");
  }
  
  delay(2000);
 
  
}
